@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2001-2003 Peter J Jones (pjones@pmade.org)
+ * Copyright (C) 2013 Vaclav Slavik <vslavik@gmail.com>
  * All Rights Reserved
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +43,8 @@
 // xmlwrapp includes
 #include "xmlwrapp/init.h"
 #include "xmlwrapp/node.h"
+#include "xmlwrapp/export.h"
+#include "xmlwrapp/errors.h"
 
 // standard includes
 #include <iosfwd>
@@ -64,18 +67,20 @@ namespace xml
 {
 
 // forward declarations
+class schema;
 class tree_parser;
 
 namespace impl
 {
 struct doc_impl;
+struct xpath_context_impl;
 }
 
 /**
     The xml::document class is used to hold the XML tree and various bits of
     information about it.
  */
-class document
+class XMLWRAPP_API document
 {
 public:
     /// size type
@@ -84,8 +89,6 @@ public:
     /**
         Create a new XML document with the default settings. The new document
         will contain a root node with a name of "blank".
-
-        @author Peter Jones
      */
     document();
 
@@ -94,35 +97,66 @@ public:
         given text.
 
         @param root_name What to set the name of the root element to.
-        @author Peter Jones
+
+        @deprecated Use `xml::document(xml::node(root_name))` constructor instead.
      */
+    XMLWRAPP_DEPRECATED("use xml::document(xml::node(root_name)) instead")
     explicit document(const char *root_name);
 
     /**
         Create a new XML document and set the root node.
 
         @param n The node to use as the root node. n will be copied.
-        @author Peter Jones
      */
     explicit document(const node& n);
 
+    /**
+        Load XML document from given file.
+
+        Errors are handled by @a on_error handler; if you pass
+        xml::throw_on_error, xml::exception is thrown on errors. If there's a
+        fatal error that prevents the document from being loaded and the error
+        handler doesn't throw an exception, the constructor will throw
+        xml::exception anyway.
+
+        @param filename The name of the file to parse.
+        @param on_error Handler called to process errors and warnings.
+
+        @since 0.7.0
+     */
+    explicit document(const char *filename, error_handler& on_error);
+
+    /**
+        Load XML document from given data.
+
+        Errors are handled by @a on_error handler; by default, xml::exception
+        is thrown on errors. If there's a fatal error that prevents the document
+        from being loaded and the error handler doesn't throw an exception, the
+        constructor will throw xml::exception anyway.
+
+        @param data The XML data to parse.
+        @param size The size of the XML data to parse.
+        @param on_error Handler called to process errors and warnings.
+
+        @since 0.7.0
+     */
+    explicit document(const char *data, size_type size, error_handler& on_error = throw_on_error);
+
     /**
         Copy construct a new XML document. The new document will be an exact
         copy of the original.
 
         @param other The other document object to copy from.
-        @author Peter Jones
      */
     document(const document& other);
 
     /**
         Copy another document object into this one using the assignment
         operator. This document object will be an exact copy of the other
-        document after the assignement.
+        document after the assignment.
 
         @param other The document to copy from.
         @return *this.
-        @author Peter Jones
      */
     document& operator=(const document& other);
 
@@ -130,14 +164,11 @@ public:
         Swap one xml::document object for another.
 
         @param other The other document to swap
-        @author Peter Jones
      */
     void swap(document& other);
 
     /**
         Clean up after an XML document object.
-
-        @author Peter Jones
      */
     ~document();
 
@@ -148,7 +179,6 @@ public:
         tree!
 
         @return A const reference to the root node.
-        @author Peter Jones
      */
     const node& get_root_node() const;
 
@@ -159,7 +189,6 @@ public:
         tree!
 
         @return A reference to the root node.
-        @author Peter Jones
      */
     node& get_root_node();
 
@@ -168,7 +197,6 @@ public:
         in the document object.
 
         @param n The new root node to use.
-        @author Peter Jones
      */
     void set_root_node(const node& n);
 
@@ -178,7 +206,6 @@ public:
         version from the XML processing instruction.
 
         @return The XML version string for this document.
-        @author Peter Jones
      */
     const std::string& get_version() const;
 
@@ -187,7 +214,6 @@ public:
         will be used when generating the XML output.
 
         @param version The version string to use, like "1.0".
-        @author Peter Jones
      */
     void set_version(const char *version);
 
@@ -196,7 +222,6 @@ public:
         ISO-8859-1.
 
         @return The encoding string.
-        @author Peter Jones
      */
     const std::string& get_encoding() const;
 
@@ -205,8 +230,6 @@ public:
         to ISO-8859-1.
 
         @param encoding The XML encoding to use.
-        @author Peter Jones
-        @author Dmitriy Nikitinskiy
      */
     void set_encoding(const char *encoding);
 
@@ -217,7 +240,6 @@ public:
 
         @return True if this document is standalone.
         @return False if this document is not standalone.
-        @author Peter Jones
      */
     bool get_is_standalone() const;
 
@@ -226,13 +248,12 @@ public:
         correct processing instruction.
 
         @param sa What to set the standalone flag to.
-        @author Peter Jones
      */
     void set_is_standalone(bool sa);
 
     /**
         Walk through the document and expand <xi:include> elements. For more
-        information, please see the w3c recomendation for XInclude.
+        information, please see the w3c recommendation for XInclude.
         http://www.w3.org/2001/XInclude.
 
         The return value of this function may change to int after a bug has
@@ -240,8 +261,6 @@ public:
 
         @return False if there was an error with substitutions.
         @return True if there were no errors (with or without substitutions).
-        @author Peter Jones
-        @author Daniel Evison
      */
     bool process_xinclude();
 
@@ -251,7 +270,6 @@ public:
 
         @return True if this document has an internal subset.
         @return False otherwise.
-        @author Peter Jones
      */
     bool has_internal_subset() const;
 
@@ -261,7 +279,6 @@ public:
 
         @return True if this document has an external subset.
         @return False otherwise.
-        @author Peter Jones
      */
     bool has_external_subset() const;
 
@@ -276,7 +293,6 @@ public:
 
         @return True if the document is valid.
         @return False if there was a problem with the DTD or XML doc.
-        @author Peter Jones
      */
     bool validate();
 
@@ -295,7 +311,6 @@ public:
         @param dtdname A filename or URL for the DTD to use.
         @return True if the document is valid.
         @return False if there was a problem with the DTD or XML doc.
-        @author Peter Jones
      */
     bool validate(const char *dtdname);
 
@@ -306,7 +321,6 @@ public:
         there are, including processing instructions, comments, etc.
 
         @return The number of children nodes that this document has.
-        @author Peter Jones
      */
     size_type size() const;
 
@@ -317,7 +331,6 @@ public:
 
         @return A xml::node::iterator that points to the first child node.
         @return An end iterator if there are no children in this document
-        @author Peter Jones
      */
     node::iterator begin();
 
@@ -328,7 +341,6 @@ public:
 
         @return A xml::node::const_iterator that points to the first child node.
         @return An end const_iterator if there are no children in this document.
-        @author Peter Jones
      */
     node::const_iterator begin() const;
 
@@ -337,7 +349,6 @@ public:
         document.
 
         @return An end xml::node::iterator.
-        @author Peter Jones
      */
     node::iterator end();
 
@@ -346,7 +357,6 @@ public:
         this document.
 
         @return An end xml::node::const_iterator.
-        @author Peter Jones
      */
     node::const_iterator end() const;
 
@@ -358,7 +368,6 @@ public:
         be thrown.
 
         @param child The child xml::node to add.
-        @author Peter Jones
      */
     void push_back (const node &child);
 
@@ -373,7 +382,6 @@ public:
         @param n The node to insert as a child of this document.
         @return An iterator that points to the newly inserted node.
         @see xml::document::push_back
-        @author Peter Jones
      */
     node::iterator insert (const node &n);
 
@@ -388,7 +396,6 @@ public:
         @param n The node to insert as a child of this document.
         @return An iterator that points to the newly inserted node.
         @see xml::document::push_back
-        @author Peter Jones
      */
     node::iterator insert(node::iterator position, const node &n);
 
@@ -407,7 +414,6 @@ public:
         @param new_node The node to put in old_node's place.
         @return An iterator that points to the new node.
         @see xml::document::push_back
-        @author Peter Jones
      */
     node::iterator replace(node::iterator old_node, const node& new_node);
 
@@ -424,12 +430,11 @@ public:
         @param to_erase An iterator that points to the node to be erased.
         @return An iterator that points to the node after the one being erased.
         @see xml::document::push_back
-        @author Peter Jones
      */
     node::iterator erase(node::iterator to_erase);
 
     /**
-        Erase all nodes in the given range, from frist to last. This will
+        Erase all nodes in the given range, from first to last. This will
         invalidate any iterators that point to the nodes to be erased, or any
         pointers or references to those nodes.
 
@@ -441,7 +446,6 @@ public:
         @param last An iterator that points one past the last node to erase. Think xml::node::end().
         @return An iterator that points to the node after the last one being erased.
         @see xml::document::push_back
-        @author Peter Jones
      */
     node::iterator erase(node::iterator first, node::iterator last);
 
@@ -449,23 +453,36 @@ public:
         Convert the XML document tree into XML text data and place it into
         the given string.
 
+        Any errors occurring while converting the document to string are passed
+        to @a on_error handler. By default, an exception will be thrown if
+        anything goes wrong.
+
         @param s The string to place the XML text data.
-        @author Peter Jones
+        @param on_error Handler called to process errors and warnings (new
+            since 0.8.0).
      */
-    void save_to_string(std::string& s) const;
+    void save_to_string(std::string& s, error_handler& on_error = throw_on_error) const;
 
     /**
         Convert the XML document tree into XML text data and place it into
         the given filename.
 
+        This function throws an exception if saving fails for any reason by
+        default and allows to customize this behaviour by passing a non-default
+        @a on_error handler.
+
         @param filename The name of the file to place the XML text data into.
+        @param on_error Handler called to process errors and warnings (new
+            since 0.8.0).
         @param compression_level 0 is no compression, 1-9 allowed, where 1 is
                                  for better speed, and 9 is for smaller size
         @return True if the data was saved successfully.
-        @return False otherwise.
-        @author Peter Jones
+        @return False otherwise (notice that this is only possible if a custom
+            error handler not throwing on error is specified).
      */
-    bool save_to_file(const char *filename, int compression_level = 0) const;
+    bool save_to_file(const char *filename,
+                      int compression_level = 0,
+                      error_handler& on_error = throw_on_error) const;
 
     /**
         Convert the XML document tree into XML text data and then insert it
@@ -474,9 +491,8 @@ public:
         @param stream The stream to insert the XML into.
         @param doc The document to insert.
         @return The stream from the first parameter.
-        @author Peter Jones
      */
-    friend std::ostream& operator<< (std::ostream &stream, const document &doc);
+    friend XMLWRAPP_API std::ostream& operator<< (std::ostream &stream, const document &doc);
 
 private:
     impl::doc_impl *pimpl_;
@@ -488,7 +504,9 @@ private:
     void* release_doc_data();
 
     friend class tree_parser;
+    friend class schema;
     friend class xslt::stylesheet;
+    friend struct impl::xpath_context_impl;
 };
 
 } // namespace xml
