@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2003 Peter J Jones (pjones@pmade.org)
- *               2009      Vaclav Slavik <vslavik@fastmail.fm>
+ *               2009      Vaclav Slavik <vslavik@gmail.com>
  * All Rights Reserved
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,7 @@
 
 // xmlwrapp includes
 #include "xmlwrapp/init.h"
+#include "xmlwrapp/export.h"
 
 // hidden stuff
 #include "xmlwrapp/_cbfo.h"
@@ -68,8 +69,10 @@ struct node_impl;
 struct doc_impl;
 struct nipimpl;
 struct node_cmp;
+struct xpath_context_impl;
 }
 
+
 /**
     The xml::node class is used to hold information about one XML node.
 
@@ -82,7 +85,7 @@ struct node_cmp;
     ANY operation to the xml::node. If you need the data to stick around a
     little longer you should put it inside a std::string.
  */
-class node
+class XMLWRAPP_API node
 {
 public:
     /// size type
@@ -165,8 +168,6 @@ public:
 
     /**
         Construct a new blank xml::node.
-
-        @author Peter Jones
      */
     node();
 
@@ -174,17 +175,17 @@ public:
         Construct a new xml::node and set the name of the node.
 
         @param name The name of the new node.
-        @author Peter Jones
      */
     explicit node(const char *name);
 
     /**
-        Construct a new xml::node given a name and content. The content will
-        be used to create a new child text node.
+        Construct a new xml::node given a name and content.
+
+        The content, if it's not an empty string, will be used to create a new
+        child text node.
 
         @param name The name of the new element.
         @param content The text that will be used to create a child node.
-        @author Peter Jones
      */
     node(const char *name, const char *content);
 
@@ -199,7 +200,6 @@ public:
 
         @param cdata_info A cdata struct that tells xml::node what the
                           content will be.
-        @author Peter Jones
      */
     explicit node(cdata cdata_info);
 
@@ -213,7 +213,6 @@ public:
         @endcode
 
         @param comment_info A comment struct that tells xml::node what the comment will be.
-        @author Peter Jones
      */
     explicit node(comment comment_info);
 
@@ -228,7 +227,6 @@ public:
         @endcode
 
         @param pi_info A pi struct that tells xml::node what the name and contents of the XML PI are.
-        @author Peter Jones
      */
     explicit node(pi pi_info);
 
@@ -242,7 +240,6 @@ public:
         @endcode
 
         @param text_info A text struct that tells xml::node what the text will be.
-        @author Vaclav Slavik
      */
     explicit node(text text_info);
 
@@ -250,7 +247,6 @@ public:
         Construct a new xml::node by copying another xml::node.
 
         @param other The other node to copy.
-        @author Peter Jones
      */
     node(const node& other);
 
@@ -259,14 +255,11 @@ public:
 
         @param other The other node to copy.
         @return A reference to this node.
-        @author Peter Jones
      */
     node& operator=(const node& other);
 
     /**
         Class destructor
-
-        @author Peter Jones
      */
     ~node();
 
@@ -274,7 +267,6 @@ public:
         Set the name of this xml::node.
 
         @param name The new name for this xml::node.
-        @author Peter Jones
      */
     void set_name(const char *name);
 
@@ -285,7 +277,6 @@ public:
         Feedback is welcome.
 
         @return The name of this node.
-        @author Peter Jones
      */
     const char* get_name() const;
 
@@ -295,10 +286,36 @@ public:
         with one text node set to the given string.
 
         @param content The content of the text node.
-        @author Peter Jones
+
+        @note @a content is supposed to be a piece of XML CDATA, so it allows
+              entity references, but XML special chars need to be escaped
+              first. In particular, the '&' character @em must be escaped
+              as "&amp;" unless it's part of entity reference. Not escaping
+              @a content may result in truncation of data. Use
+              set_text_content() if @a content may contain special characters.
+
+        @see set_text_content()
      */
     void set_content(const char *content);
 
+    /**
+        Set the content of a node to given text.
+
+        In contrast to set_content(), @a content is raw text, so unescaped XML
+        special chars are allowed and entity references are not supported.
+
+        If this node is an element node, this function will remove all of its
+        children nodes and replace them with one text node set to the given
+        string.
+
+        @param content The content text.
+
+        @see set_content()
+
+        @since 0.7.0
+     */
+    void set_text_content(const char *content);
+
     /**
         Get the content for this text node. If this node is not a text node
         but it has children nodes that are text nodes, the contents of those
@@ -309,7 +326,6 @@ public:
         Feedback is welcome.
 
         @return The content or 0.
-        @author Peter Jones
      */
     const char* get_content() const;
 
@@ -318,7 +334,6 @@ public:
         can and cannot do with it.
 
         @return The node's type.
-        @author Peter Jones
      */
     node_type get_type() const;
 
@@ -328,7 +343,6 @@ public:
         to this returned object, to prevent a copy.
 
         @return The xml::attributes object for this node.
-        @author Peter Jones
      */
     xml::attributes& get_attributes();
 
@@ -338,7 +352,6 @@ public:
         returned object, to prevent a copy.
 
         @return The xml::attributes object for this node.
-        @author Peter Jones
      */
     const xml::attributes& get_attributes() const;
 
@@ -347,17 +360,32 @@ public:
 
         @return The namespace of this node or NULL if no namespace is
                 associated.
-        @author Vaclav Slavik
         @since  0.6.0
      */
     const char* get_namespace() const;
 
     /**
-        Find out if this node is a text node or sometiming like a text node,
+        Set the default namespace of this xml::node.
+
+        If the default namespace is already set on this node, it is changed.
+
+        Example:
+
+        @code
+        // n is <foo/>
+        n.set_namespace("http://example.com")
+        // n is <foo xmlns="http://example.com"/>
+        @endcode
+
+        @since 0.8.0
+     */
+    void set_namespace(const std::string& href);
+
+    /**
+        Find out if this node is a text node or something like a text node,
         CDATA for example.
 
         @return True if this node is a text node; false otherwise.
-        @author Peter Jones
      */
     bool is_text() const;
 
@@ -365,7 +393,6 @@ public:
         Add a child xml::node to this node.
 
         @param child The child xml::node to add.
-        @author Peter Jones
      */
     void push_back(const node& child);
 
@@ -373,10 +400,28 @@ public:
         Swap this node with another one.
 
         @param other The other node to swap with.
-        @author Peter Jones
      */
     void swap(node& other);
 
+    /**
+        Move this node under another parent.
+
+        This node will become the last child of @a new_parent. Notice that this
+        node must not be an ancestor of @a new_parent, in particular it
+        shouldn't be the document root.
+
+        Currently this method can only be used to move nodes inside the same
+        document.
+
+        All iterators pointing to this node are invalidated after the move.
+
+        @param new_parent The new parent for the node.
+
+        @since 0.8.0
+     */
+    void move_under(node& new_parent);
+
+
     class const_iterator; // forward declaration
 
     /**
@@ -407,11 +452,6 @@ public:
         /// postfix increment (avoid if possible for better performance)
         iterator  operator++ (int);
 
-        bool operator==(const iterator& other) const
-            { return get_raw_node() == other.get_raw_node(); }
-        bool operator!=(const iterator& other) const
-            { return !(*this == other); }
-
     private:
         impl::nipimpl *pimpl_;
 
@@ -422,6 +462,7 @@ public:
         friend class node;
         friend class document;
         friend class const_iterator;
+        friend bool XMLWRAPP_API operator==(const iterator& lhs, const iterator& rhs);
     };
 
     /**
@@ -453,11 +494,6 @@ public:
         /// postfix increment (avoid if possible for better performance)
         const_iterator  operator++ (int);
 
-        bool operator==(const const_iterator& other) const
-            { return get_raw_node() == other.get_raw_node(); }
-        bool operator!=(const const_iterator& other) const
-            { return !(*this == other); }
-
     private:
         impl::nipimpl *pimpl_;
 
@@ -467,15 +503,15 @@ public:
 
         friend class document;
         friend class node;
+        friend bool XMLWRAPP_API operator==(const const_iterator& lhs, const const_iterator& rhs);
     };
 
     /**
-        Returns the number of childer this nodes has. If you just want to
+        Returns the number of children this nodes has. If you just want to
         know how if this node has children or not, you should use
         xml::node::empty() instead.
 
         @return The number of children this node has.
-        @author Peter Jones
      */
     size_type size() const;
 
@@ -485,7 +521,6 @@ public:
 
         @return True if this node DOES NOT have any children.
         @return False if this node does have children.
-        @author Peter Jones
      */
     bool empty() const;
 
@@ -493,7 +528,6 @@ public:
         Get an iterator that points to the beginning of this node's children.
 
         @return An iterator that points to the beginning of the children.
-        @author Peter Jones
      */
     iterator begin();
 
@@ -502,7 +536,6 @@ public:
         children.
 
         @return A const_iterator that points to the beginning of the children.
-        @author Peter Jones
      */
     const_iterator begin() const;
 
@@ -510,7 +543,6 @@ public:
         Get an iterator that points one past the last child for this node.
 
         @return A "one past the end" iterator.
-        @author Peter Jones
      */
     iterator end() { return iterator(); }
 
@@ -519,7 +551,6 @@ public:
         node.
 
         @return A "one past the end" const_iterator
-        @author Peter Jones
      */
     const_iterator end() const { return const_iterator(); }
 
@@ -527,7 +558,6 @@ public:
         Get an iterator that points back at this node.
 
         @return An iterator that points at this node.
-        @author Peter Jones
      */
     iterator self();
 
@@ -535,7 +565,6 @@ public:
         Get a const_iterator that points back at this node.
 
         @return A const_iterator that points at this node.
-        @author Peter Jones
      */
     const_iterator self() const;
 
@@ -546,7 +575,6 @@ public:
 
         @return An iterator that points to this nodes parent.
         @return If no parent, returns the same iterator that xml::node::end() returns.
-        @author Peter Jones
      */
     iterator parent();
 
@@ -557,7 +585,6 @@ public:
 
         @return A const_iterator that points to this nodes parent.
         @return If no parent, returns the same const_iterator that xml::node::end() returns.
-        @author Peter Jones
      */
     const_iterator parent() const;
 
@@ -573,7 +600,6 @@ public:
         @param name The name of the node you want to find.
         @return An iterator that points to the node if found.
         @return An end() iterator if the node was not found.
-        @author Peter Jones
 
         @see elements(const char*), find(const char*, iterator)
      */
@@ -591,7 +617,6 @@ public:
         @param name The name of the node you want to find.
         @return A const_iterator that points to the node if found.
         @return An end() const_iterator if the node was not found.
-        @author Peter Jones
 
         @see elements(const char*) const,
              find(const char*, const_iterator) const
@@ -612,7 +637,6 @@ public:
         @param start Where to begin the search.
         @return An iterator that points to the node if found.
         @return An end() iterator if the node was not found.
-        @author Peter Jones
 
         @see elements(const char*)
      */
@@ -632,7 +656,6 @@ public:
         @param start Where to begin the search.
         @return A const_iterator that points to the node if found.
         @return An end() const_iterator if the node was not found.
-        @author Peter Jones
 
         @see elements(const char*) const
      */
@@ -652,7 +675,6 @@ public:
         @endcode
 
         @return View with all child elements or empty view if there aren't any.
-        @author Vaclav Slavik
         @since  0.6.0
 
         @see nodes_view
@@ -675,7 +697,6 @@ public:
         @endcode
 
         @return View with all child elements or empty view if there aren't any.
-        @author Vaclav Slavik
         @since  0.6.0
 
         @see const_nodes_view
@@ -688,7 +709,7 @@ public:
 
         Example:
         @code
-        xml::nodes_view persons(root.elements("person"));
+        xml::nodes_view view(root.elements("person"));
         for (xml::nodes_view::iterator i = view.begin(); i != view.end(); ++i)
         {
           ...
@@ -697,7 +718,6 @@ public:
 
         @param  name Name of the elements to return.
         @return View that contains only elements @a name.
-        @author Vaclav Slavik
         @since  0.6.0
      */
     nodes_view elements(const char *name);
@@ -708,7 +728,7 @@ public:
 
         Example:
         @code
-        xml::const_nodes_view persons(root.elements("person"));
+        xml::const_nodes_view view(root.elements("person"));
         for (xml::const_nodes_view::const_iterator i = view.begin();
              i != view.end();
              ++i)
@@ -719,7 +739,6 @@ public:
 
         @param  name Name of the elements to return.
         @return View that contains only elements @a name.
-        @author Vaclav Slavik
         @since  0.6.0
      */
     const_nodes_view elements(const char *name) const;
@@ -731,7 +750,6 @@ public:
 
         @param n The node to insert as a child of this node.
         @return An iterator that points to the newly inserted node.
-        @author Peter Jones
      */
     iterator insert(const node& n);
 
@@ -742,7 +760,6 @@ public:
         @param position An iterator that points to the location where the new node should be inserted (before it).
         @param n The node to insert as a child of this node.
         @return An iterator that points to the newly inserted node.
-        @author Peter Jones
      */
     iterator insert(const iterator& position, const node& n);
 
@@ -756,7 +773,6 @@ public:
         @param old_node An iterator that points to the node that should be removed.
         @param new_node The node to put in old_node's place.
         @return An iterator that points to the new node.
-        @author Peter Jones
      */
     iterator replace(const iterator& old_node, const node& new_node);
 
@@ -768,20 +784,17 @@ public:
 
         @param to_erase An iterator that points to the node to be erased.
         @return An iterator that points to the node after the one being erased.
-        @author Peter Jones
-        @author Gary A. Passero
      */
     iterator erase(const iterator& to_erase);
 
     /**
-        Erase all nodes in the given range, from frist to last. This will
+        Erase all nodes in the given range, from first to last. This will
         invalidate any iterators that point to the nodes to be erased, or any
         pointers or references to those nodes.
 
         @param first The first node in the range to be removed.
         @param last An iterator that points one past the last node to erase. Think xml::node::end().
         @return An iterator that points to the node after the last one being erased.
-        @author Peter Jones
      */
     iterator erase(iterator first, const iterator& last);
 
@@ -793,12 +806,18 @@ public:
 
         @param name The name of nodes to remove.
         @return The number of nodes removed.
-        @author Peter Jones
      */
     size_type erase(const char *name);
 
     /**
-        Sort all the children nodes of this node using one of thier
+        Erases all children nodes.
+
+        @since 0.7.0
+     */
+    void clear();
+
+    /**
+        Sort all the children nodes of this node using one of their
         attributes. Only nodes that are of xml::node::type_element will be
         sorted, and they must have the given node_name.
 
@@ -807,7 +826,6 @@ public:
 
         @param node_name The name of the nodes to sort.
         @param attr_name The attribute to sort on.
-        @author Peter Jones
      */
     void sort(const char *node_name, const char *attr_name);
 
@@ -817,19 +835,26 @@ public:
         sorting.
 
         @param compare The binary function object to call in order to sort all child nodes.
-        @author Peter Jones
      */
     template <typename T> void sort (T compare)
         { impl::sort_callback<T> cb(compare); sort_fo(cb); }
 
+    /**
+        Convert the node and all its children into XML text and return the
+        string containing them.
+
+        @since 0.8.0
+     */
+    std::string node_to_string() const;
+
     /**
         Convert the node and all its children into XML text and set the given
         string to that text.
 
         @param xml The string to set the node's XML data to.
-        @author Peter Jones
      */
-    void node_to_string(std::string& xml) const;
+    void node_to_string(std::string& xml) const
+        { xml = node_to_string(); }
 
     /**
         Write a node and all of its children to the given stream.
@@ -837,9 +862,8 @@ public:
         @param stream The stream to write the node as XML.
         @param n The node to write to the stream.
         @return The stream.
-        @author Peter Jones
      */
-    friend std::ostream& operator<< (std::ostream &stream, const node &n);
+    friend XMLWRAPP_API std::ostream& operator<< (std::ostream &stream, const node &n);
 
 private:
     impl::node_impl *pimpl_;
@@ -858,8 +882,26 @@ private:
     friend class document;
     friend struct impl::doc_impl;
     friend struct impl::node_cmp;
+    friend class xml::const_nodes_view;
+    friend struct impl::xpath_context_impl;
 };
 
+// Comparison operators for xml::node iterators
+
+inline bool XMLWRAPP_API operator==(const node::iterator& lhs,
+                                    const node::iterator& rhs)
+    { return lhs.get_raw_node() == rhs.get_raw_node(); }
+inline bool XMLWRAPP_API operator!=(const node::iterator& lhs,
+                                    const node::iterator& rhs)
+    { return !(lhs == rhs); }
+
+inline bool XMLWRAPP_API operator==(const node::const_iterator& lhs,
+                                    const node::const_iterator& rhs)
+    { return lhs.get_raw_node() == rhs.get_raw_node(); }
+inline bool XMLWRAPP_API operator!=(const node::const_iterator& lhs,
+                                    const node::const_iterator& rhs)
+    { return !(lhs == rhs); }
+
 } // namespace xml
 
 #endif // _xmlwrapp_node_h_
