@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2003 Peter J Jones (pjones@pmade.org)
- *               2009      Vaclav Slavik <vslavik@fastmail.fm>
+ *               2009      Vaclav Slavik <vslavik@gmail.com>
  * All Rights Reserved
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,8 @@
 #include "xmlwrapp/node.h"
 #include "xmlwrapp/nodes_view.h"
 #include "xmlwrapp/attributes.h"
+#include "xmlwrapp/errors.h"
+#include "cpp11.h"
 #include "utility.h"
 #include "ait_impl.h"
 #include "node_manip.h"
@@ -268,7 +270,7 @@ node::node(int)
 
 node::node()
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     pimpl_->xmlnode_ = xmlNewNode(0, reinterpret_cast<const xmlChar*>("blank"));
     if (!pimpl_->xmlnode_)
@@ -280,7 +282,7 @@ node::node()
 
 node::node (const char *name)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     pimpl_->xmlnode_ = xmlNewNode(0, reinterpret_cast<const xmlChar*>(name));
     if (!pimpl_->xmlnode_)
@@ -292,20 +294,23 @@ node::node (const char *name)
 
 node::node (const char *name, const char *content)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     pimpl_->xmlnode_ = xmlNewNode(0, reinterpret_cast<const xmlChar*>(name));
     if (!pimpl_->xmlnode_)
         throw std::bad_alloc();
 
-    xmlNodePtr content_node = xmlNewText(reinterpret_cast<const xmlChar*>(content));
-    if (!content_node)
-        throw std::bad_alloc();
-
-    if (!xmlAddChild(pimpl_->xmlnode_, content_node))
+    if (std::strlen(content))
     {
-        xmlFreeNode(content_node);
-        throw std::bad_alloc();
+        xmlNodePtr content_node = xmlNewText(reinterpret_cast<const xmlChar*>(content));
+        if (!content_node)
+            throw std::bad_alloc();
+
+        if (!xmlAddChild(pimpl_->xmlnode_, content_node))
+        {
+            xmlFreeNode(content_node);
+            throw std::bad_alloc();
+        }
     }
 
     ap.release();
@@ -314,7 +319,7 @@ node::node (const char *name, const char *content)
 
 node::node(cdata cdata_info)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     if ( (pimpl_->xmlnode_ = xmlNewCDataBlock(0, reinterpret_cast<const xmlChar*>(cdata_info.t), std::strlen(cdata_info.t))) == 0)
     {
@@ -327,7 +332,7 @@ node::node(cdata cdata_info)
 
 node::node(comment comment_info)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     if ( (pimpl_->xmlnode_ =  xmlNewComment(reinterpret_cast<const xmlChar*>(comment_info.t))) == 0)
     {
@@ -340,7 +345,7 @@ node::node(comment comment_info)
 
 node::node(pi pi_info)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     if ( (pimpl_->xmlnode_ = xmlNewPI(reinterpret_cast<const xmlChar*>(pi_info.n), reinterpret_cast<const xmlChar*>(pi_info.c))) == 0)
     {
@@ -353,7 +358,7 @@ node::node(pi pi_info)
 
 node::node(text text_info)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     if ( (pimpl_->xmlnode_ =  xmlNewText(reinterpret_cast<const xmlChar*>(text_info.t))) == 0)
     {
@@ -366,7 +371,7 @@ node::node(text text_info)
 
 node::node(const node& other)
 {
-    std::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
+    xml::impl::auto_ptr<node_impl> ap(pimpl_ = new node_impl);
 
     pimpl_->xmlnode_ = xmlCopyNode(other.pimpl_->xmlnode_, 1);
     if (!pimpl_->xmlnode_)
@@ -390,6 +395,58 @@ void node::swap(node& other)
 }
 
 
+void node::move_under(node& new_parent)
+{
+    xmlNodePtr& this_node = pimpl_->xmlnode_;
+    xmlNodePtr& new_parent_node = new_parent.pimpl_->xmlnode_;
+
+    if (this_node->parent == new_parent_node)
+        return;
+
+    // If we wanted to support inter document moves, we would need to deal with
+    // the namespaces, e.g. we could need to define the namespace of this node
+    // in the new document. While this should be possible, currently this is
+    // not implemented.
+    if (this_node->doc != new_parent_node->doc)
+        throw xml::exception("moving nodes between documents not supported");
+
+    // Check that we're not moving the node under itself, loops are not allowed
+    // in XML trees.
+    for (xmlNodePtr p = new_parent_node; p; p = p->parent)
+    {
+        if (p == this_node)
+            throw xml::exception("can't move a node under itself");
+    }
+
+    // Note that this must be non-NULL as we're not the tree root, otherwise
+    // the new parent would have been our child and we would have thrown above.
+    xmlNodePtr& old_parent_node = this_node->parent;
+
+    // Remove the node from the old parent children list.
+    if (this_node->prev)
+        this_node->prev->next = this_node->next;
+    else
+        old_parent_node->children = this_node->next;
+
+    if (this_node->next)
+        this_node->next->prev = this_node->prev;
+    else
+        old_parent_node->last = this_node->prev;
+
+    // Update the pointers inside this node itself.
+    this_node->parent = new_parent_node;
+    this_node->prev = new_parent_node->last;
+    this_node->next = NULL;
+
+    // And update the new parent too.
+    if (this_node->prev)
+        this_node->prev->next = this_node;
+    else
+        new_parent_node->children = this_node;
+    new_parent_node->last = this_node;
+}
+
+
 node::~node()
 {
     delete pimpl_;
@@ -435,6 +492,16 @@ void node::set_content(const char *content)
 }
 
 
+void node::set_text_content(const char *content)
+{
+    xmlChar *escaped = xmlEncodeSpecialChars(pimpl_->xmlnode_->doc,
+                                             reinterpret_cast<const xmlChar*>(content));
+    xmlNodeSetContent(pimpl_->xmlnode_, escaped);
+    if ( escaped )
+        xmlFree(escaped);
+}
+
+
 const char* node::get_content() const
 {
     xmlchar_helper content(xmlNodeGetContent(pimpl_->xmlnode_));
@@ -477,7 +544,7 @@ xml::attributes& node::get_attributes()
 {
     if (pimpl_->xmlnode_->type != XML_ELEMENT_NODE)
     {
-        throw std::runtime_error("get_attributes called on non-element node");
+        throw xml::exception("get_attributes called on non-element node");
     }
 
     pimpl_->attrs_.set_data(pimpl_->xmlnode_);
@@ -489,7 +556,7 @@ const xml::attributes& node::get_attributes() const
 {
     if (pimpl_->xmlnode_->type != XML_ELEMENT_NODE)
     {
-        throw std::runtime_error("get_attributes called on non-element node");
+        throw xml::exception("get_attributes called on non-element node");
     }
 
     pimpl_->attrs_.set_data(pimpl_->xmlnode_);
@@ -505,6 +572,36 @@ const char *node::get_namespace() const
 }
 
 
+void node::set_namespace(const std::string& href)
+{
+    const xmlChar *xmlHref = reinterpret_cast<const xmlChar*>(href.c_str());
+
+    if (pimpl_->xmlnode_->type != XML_ELEMENT_NODE)
+        throw xml::exception("set_namespace called on non-element node");
+
+    xmlNsPtr ns = xmlNewNs(pimpl_->xmlnode_, xmlHref, NULL);
+
+    if ( !ns )
+    {
+        // Looks like the default namespace already exists on this node,
+        // we must change its URI.
+        for ( ns = pimpl_->xmlnode_->nsDef; ns; ns = ns->next )
+        {
+            if ( ns->prefix == NULL )
+            {
+                xmlFree((void*)ns->href);
+                ns->href = xmlStrdup(xmlHref);
+                break;
+            }
+        }
+        if ( !ns )
+            throw xml::exception("set_namespace failed to create namespace object");
+    }
+
+    xmlSetNs(pimpl_->xmlnode_, ns);
+}
+
+
 bool node::is_text() const
 {
     return xmlNodeIsText(pimpl_->xmlnode_) != 0;
@@ -519,7 +616,8 @@ void node::push_back (const node &child)
 
 node::size_type node::size() const
 {
-    return std::distance(begin(), end());
+    using namespace std;
+    return distance(begin(), end());
 }
 
 
@@ -590,7 +688,7 @@ node::const_iterator node::find(const char *name) const
 node::iterator node::find(const char *name, const iterator& start)
 {
     xmlNodePtr n = static_cast<xmlNodePtr>(start.get_raw_node());
-    if ( (n = find_element(name, n)))
+    if ((n = find_element(name, n)) != NULL)
         return iterator(n);
     return end();
 }
@@ -599,7 +697,7 @@ node::iterator node::find(const char *name, const iterator& start)
 node::const_iterator node::find(const char *name, const const_iterator& start) const
 {
     xmlNodePtr n = static_cast<xmlNodePtr>(start.get_raw_node());
-    if ( (n = find_element(name, n)))
+    if ((n = find_element(name, n)) != NULL)
         return const_iterator(n);
     return end();
 }
@@ -688,6 +786,19 @@ node::size_type node::erase(const char *name)
 }
 
 
+void node::clear()
+{
+    xmlNodePtr n = pimpl_->xmlnode_;
+
+    if ( !n->children )
+        return;
+
+    xmlFreeNodeList(n->children);
+    n->children =
+    n->last = NULL;
+}
+
+
 void node::sort(const char *node_name, const char *attr_name)
 {
     xmlNodePtr i(pimpl_->xmlnode_->children), next(0);
@@ -739,7 +850,7 @@ void node::sort_fo(cbfo_node_compare& cb)
 }
 
 
-void node::node_to_string(std::string& xml) const
+std::string node::node_to_string() const
 {
     node2doc n2d(pimpl_->xmlnode_);
     xmlDocPtr doc = n2d.get_doc();
@@ -750,16 +861,16 @@ void node::node_to_string(std::string& xml) const
     xmlDocDumpFormatMemory(doc, &xml_string, &xml_string_length, 1);
 
     xmlchar_helper helper(xml_string);
+    std::string xml;
     if (xml_string_length)
         xml.assign(helper.get(), xml_string_length);
+    return xml;
 }
 
 
 std::ostream& operator<<(std::ostream &stream, const xml::node& n)
 {
-    std::string xmldata;
-    n.node_to_string(xmldata);
-    stream << xmldata;
+    stream << n.node_to_string();
     return stream;
 }
 
